@@ -11,6 +11,7 @@ SEXP attribute_hidden
     lme4_STSym,
     lme4_VSym,
     lme4_XSym,
+    lme4_XstSym,
     lme4_ZtSym,
     lme4_devianceSym,
     lme4_dimsSym,
@@ -18,6 +19,8 @@ SEXP attribute_hidden
     lme4_etaSym,
     lme4_fixefSym,
     lme4_flistSym,
+    lme4_ghwSym,
+    lme4_ghxSym,
     lme4_gradientSym,
     lme4_iSym,
     lme4_muEtaSym,
