@@ -15,7 +15,7 @@ SEXP mer_ST_getPars(SEXP x);
 SEXP mer_ST_initialize(SEXP ST, SEXP Gp, SEXP Zt);
 SEXP mer_ST_setPars(SEXP x, SEXP pars);
 SEXP mer_create_L(SEXP CmP);
-SEXP mer_optimize(SEXP x, SEXP verb);
+SEXP mer_optimize(SEXP x);
 SEXP mer_postVar(SEXP x);
 SEXP mer_update_L(SEXP x);
 SEXP mer_update_RX(SEXP x);
@@ -23,7 +23,7 @@ SEXP mer_update_dev(SEXP x);
 SEXP mer_update_projection(SEXP x);
 SEXP mer_update_ranef(SEXP x);
 SEXP mer_update_mu(SEXP x);
-SEXP mer_update_u(SEXP x, SEXP verbP);
+SEXP mer_update_u(SEXP x);
 SEXP mer_validate(SEXP x);
 SEXP merMCMC_validate(SEXP x);
 SEXP merMCMC_VarCorr(SEXP x, SEXP typ);
