@@ -37,22 +37,23 @@ e3 <- expand(nm3)
 stopifnot(identical(sapply(e3, class),
                     c(sigma = "numeric", P = "pMatrix",
                       T = "dtCMatrix", S = "ddiMatrix"))
-          , allEQ(e3$sigma, c(sigmaML = 0.76921295))
+          , allEQ(e3$sigma, c(sigmaML = 0.70777))
           , all(e3$P@perm == outer(12*(0:2), 1:12, "+"))
           , identical(as(e3$T, "diagonalMatrix"), Diagonal(3*12))
-          , allEQ(e3$S@x, rep(c(0, 0.620071, 0.163092), each=12))
+          , allEQ(e3$S@x, rep(c(0, 0.92746, 0.23667), each=12))
           )
 
 e2 <- expand(nm2) # -> gave error!
 stopifnot(identical(sapply(e2, class),
                     c(sigma = "numeric", P = "pMatrix",
                       T = "dtCMatrix", S = "ddiMatrix"))
-          , allEQ(e2$sigma, c(sigmaML = 0.7692315))
+          , allEQ(e2$sigma, c(sigmaML = 0.70777))
           , all(e2$P@perm == outer(12*(0:2), 1:12, "+"))
-          , diag(e2$T == 1), nnzero(e2$T) == 36 + 24 + 12
+          , all(diag(e2$T == 1))
+          , nnzero(e2$T) == 36 + 24 + 12
           , allEQ(unique(e2$T@x),
-                  c(1, 0.001, 0.0564, -0.00328), tol = .009)
-          , allEQ(e2$S@x, rep(c(1e-6, 0.6198669, 0.1629919), each=12))
+                  c(1, 0.0310, 0.0909, -0.00187), tol = .009)
+          , allEQ(e2$S@x, rep(c(0.0000000, 0.9274296, 0.2366269), each=12))
           )
 
 showProc.time()
